@@ -2,7 +2,7 @@
  * config.h: limits and configuration
  * Defines all compilation-time configuration constants and size limits
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
+ * Copyright (C) 1999, 2000, 2001 VideoLAN
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@via.ecp.fr>
@@ -41,13 +41,13 @@
  *****************************************************************************/
 
 /* Program version and copyright message */
-#define VERSION_MESSAGE     "vlc 0.2.63 Urumov " \
+#define VERSION_MESSAGE     "vlc 0.2.71 Ourumov " \
                           /* "(" PROGRAM_BUILD ") (" PROGRAM_OPTIONS ")\n" */ \
-                            "Copyright 1996-2000 VideoLAN\n"
-#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.63" \
-                            " Urumov - (c)1996-2000 VideoLAN\n"
+                            "Copyright 1996-2001 VideoLAN\n"
+#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.71" \
+                            " Ourumov - (C)1996-2001 VideoLAN\n"
 
-#define VERSION             "0.2.63"
+#define VERSION             "0.2.71"
 
 /*****************************************************************************
  * General compilation options
@@ -63,22 +63,15 @@
  * and might cause problems with some very weird streams. */
 //#define MPEG2_COMPLIANT
 
-/* Define for profiling and statistics support - such informations, like FPS
- * or pictures count won't be available if it not set */
-/* #define STATS --moved in Makefile */
-
-/* Define for unthreaded version of the program FIXME: not yet implemented ??*/
-//#define NO_THREAD
-
 /*****************************************************************************
  * Debugging options - define or undefine symbols
  *****************************************************************************/
-#if DEBUG == 1
+#ifdef DEBUG
 /* General debugging support, which depends of the DEBUG define, is determined
  * in the Makefile */
 
 /* Modules specific debugging - this will produce a lot of output, but can be
- * usefull to track a bug */
+ * useful to track a bug */
 //#define DEBUG_INTF
 //#define DEBUG_INPUT
 //#define DEBUG_AUDIO
@@ -113,7 +106,7 @@
  */
 
 /* Size of the FIFO. FIFO_SIZE+1 must be a power of 2 */
-#define FIFO_SIZE                       511
+#define FIFO_SIZE                       1023
 
 /*
  * Paths
@@ -179,6 +172,7 @@
 /* Default remote server */
 #define INPUT_SERVER_VAR                "vlc_server"
 #define INPUT_SERVER_DEFAULT            "138.195.143.220"
+#define INPUT_BCAST_ADDR                "138.195.143.255"
 
 /* Default input port */
 #define INPUT_PORT_VAR                  "vlc_server_port"
@@ -189,7 +183,7 @@
 #define INPUT_BROADCAST_DEFAULT         0
 
 /*
- * Vlan method
+ * Channel method
  */
 
 /* Default network interface and environment variable */
@@ -197,14 +191,14 @@
 #define INPUT_IFACE_DEFAULT             "eth0"
 
 /* Default server and port */
-#define INPUT_VLAN_SERVER_VAR           "vlc_vlan_server"
-#define INPUT_VLAN_SERVER_DEFAULT       "138.195.139.95"
-#define INPUT_VLAN_PORT_VAR             "vlc_vlan_port"
-#define INPUT_VLAN_PORT_DEFAULT         6010
+#define INPUT_CHANNEL_SERVER_VAR           "vlc_channel_server"
+#define INPUT_CHANNEL_SERVER_DEFAULT       "138.195.139.95"
+#define INPUT_CHANNEL_PORT_VAR             "vlc_channel_port"
+#define INPUT_CHANNEL_PORT_DEFAULT         6010
 
-/* Delay between vlan changes - this is required to avoid flooding the VLAN
- * server */
-#define INPUT_VLAN_CHANGE_DELAY         (mtime_t)(5*CLOCK_FREQ)
+/* Delay between channel changes - this is required to avoid flooding the 
+ * channel server */
+#define INPUT_CHANNEL_CHANGE_DELAY         (mtime_t)(5*CLOCK_FREQ)
 
 /* Duration between the time we receive the data packet, and the time we will
  * mark it to be presented */
@@ -372,7 +366,7 @@
 #define VOUT_FB_DEV_DEFAULT             "/dev/fb0"
 
 /* The default video output window title */
-#define VOUT_TITLE                      "VideoLAN Client 0.2.63"
+#define VOUT_TITLE                      "VideoLAN Client 0.2.71"
 
 /*****************************************************************************
  * Video parser configuration
@@ -444,11 +438,11 @@
 #define INTF_WARNING_VAR                "warning_level"
 #define INTF_WARNING_DEFAULT            12
 
-/* Define to enable messages queues - disabling messages queue can be usefull
- * when debugging, since it allows messages which would not otherwise be printed,
- * due to a crash, to be printed anyway */
+/* Define to enable messages queues - disabling messages queue can be useful
+ * when debugging, since it allows messages which would not be printed
+ * due to a crash to be printed anyway */
 #ifndef DEBUG
-#define INTF_MSG_QUEUE
+//#define INTF_MSG_QUEUE
 #endif
 
 /* Format of the header for debug messages. The arguments following this header
