@@ -4,6 +4,7 @@
  * includes all common video types and constants.
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
+ * $Id: video.h,v 1.29 2001/03/21 13:42:33 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -57,8 +58,10 @@ typedef struct picture_s
     /* Picture static properties - those properties are fixed at initialization
      * and should NOT be modified */
     int             i_width;                                /* picture width */
-    int             i_height;                              /* picture height */
     int             i_chroma_width;                          /* chroma width */
+    int             i_height;                              /* picture height */
+    int             i_size;                                /* number of pels */
+    int             i_chroma_size;                  /* number of chroma pels */
 
     /* Picture dynamic properties - those properties can be changed by the
      * decoder */
