@@ -2,7 +2,7 @@
  * vout_x11.c: X11 video output display method
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: vout_x11.c,v 1.17 2001/04/01 06:21:44 sam Exp $
+ * $Id: vout_x11.c,v 1.18 2001/04/11 02:01:24 henri Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -60,6 +60,8 @@
 #include "interface.h"
 #include "intf_msg.h"
 
+#include "netutils.h"                                 /* network_ChannelJoin */
+
 #include "main.h"
 
 /*****************************************************************************
@@ -100,6 +102,9 @@ typedef struct vout_sys_s
     int                 i_ss_blanking;                      /* blanking mode */
     int                 i_ss_exposure;                      /* exposure mode */
 
+    /* Auto-hide cursor */
+    mtime_t     i_lastmoved;
+    
     /* Mouse pointer properties */
     boolean_t           b_mouse;         /* is the mouse pointer displayed ? */
 
@@ -288,6 +293,10 @@ static int vout_Init( vout_thread_t *p_vout )
     p_vout->i_bytes_per_line = p_vout->p_sys->p_ximage[0]->bytes_per_line;
     vout_SetBuffers( p_vout, p_vout->p_sys->p_ximage[ 0 ]->data,
                      p_vout->p_sys->p_ximage[ 1 ]->data );
+
+    /* Set date for autohiding cursor */
+    p_vout->p_sys->i_lastmoved = mdate();
+    
     return( 0 );
 }
 
@@ -352,6 +361,8 @@ static int vout_Manage( vout_thread_t *p_vout )
     XEvent      xevent;                                         /* X11 event */
     boolean_t   b_resized;                        /* window has been resized */
     char        i_key;                                    /* ISO Latin-1 key */
+    KeySym      x_key_symbol;
+
 
     /* Handle X11 events: ConfigureNotify events are parsed to know if the
      * output window's size changed, MapNotify and UnmapNotify to know if the
@@ -360,7 +371,8 @@ static int vout_Manage( vout_thread_t *p_vout )
     b_resized = 0;
     while( XCheckWindowEvent( p_vout->p_sys->p_display, p_vout->p_sys->window,
                               StructureNotifyMask | KeyPressMask |
-                              ButtonPressMask | ButtonReleaseMask, &xevent )
+                              ButtonPressMask | ButtonReleaseMask | 
+                              PointerMotionMask | Button1MotionMask , &xevent )
            == True )
     {
         /* ConfigureNotify event: prepare  */
@@ -394,16 +406,73 @@ static int vout_Manage( vout_thread_t *p_vout )
         /* Keyboard event */
         else if( xevent.type == KeyPress )
         {
-            if( XLookupString( &xevent.xkey, &i_key, 1, NULL, NULL ) )
+            /* We may have keys like F1 trough F12, ESC ... */
+            x_key_symbol = XKeycodeToKeysym( p_vout->p_sys->p_display,
+                                             xevent.xkey.keycode, 0 );
+            switch( x_key_symbol )
             {
-                /* FIXME: handle stuff here */
-                switch( i_key )
-                {
-                case 'q':
-                    /* FIXME: need locking ! */
-                    p_main->p_intf->b_die = 1;
-                    break;
-                }
+                 case XK_Escape:
+                     p_main->p_intf->b_die = 1;
+                     break;
+                 case XK_Menu:
+                     p_main->p_intf->b_menu_change = 1;
+                     break;
+                 default:
+                     /* "Normal Keys"
+                      * The reason why I use this instead of XK_0 is that 
+                      * with XLookupString, we don't have to care about
+                      * keymaps. */
+
+                    if( XLookupString( &xevent.xkey, &i_key, 1, NULL, NULL ) )
+                    {
+                        /* FIXME: handle stuff here */
+                        switch( i_key )
+                        {
+                        case 'q':
+                        case 'Q':
+                            p_main->p_intf->b_die = 1;
+                            break;
+                        case '0':
+                            network_ChannelJoin( 0 );
+                            break;
+                        case '1':
+                            network_ChannelJoin( 1 );
+                            break;
+                        case '2':
+                            network_ChannelJoin( 2 );
+                            break;
+                        case '3':
+                            network_ChannelJoin( 3 );
+                            break;
+                        case '4':
+                            network_ChannelJoin( 4 );
+                            break;
+                        case '5':
+                            network_ChannelJoin( 5 );
+                            break;
+                        case '6':
+                            network_ChannelJoin( 6 );
+                            break;
+                        case '7':
+                            network_ChannelJoin( 7 );
+                            break;
+                        case '8':
+                            network_ChannelJoin( 8 );
+                            break;
+                        case '9':
+                            network_ChannelJoin( 9 );
+                            break;
+                        default:
+                            if( intf_ProcessKey( p_main->p_intf, 
+                                                 (char )i_key ) )
+                            {
+                               intf_DbgMsg( "unhandled key '%c' (%i)", 
+                                            (char)i_key, i_key );
+                            }
+                            break;
+                        }
+                    }
+                break;
             }
         }
         /* Mouse click */
@@ -415,10 +484,6 @@ static int vout_Manage( vout_thread_t *p_vout )
                     /* in this part we will eventually manage
                      * clicks for DVD navigation for instance */
                     break;
-
-                case Button2:
-                    X11TogglePointer( p_vout );
-                    break;
             }
         }
         /* Mouse release */
@@ -432,6 +497,15 @@ static int vout_Manage( vout_thread_t *p_vout )
                     break;
             }
         }
+        /* Mouse move */
+        else if( xevent.type == MotionNotify )
+        {
+            p_vout->p_sys->i_lastmoved = mdate();
+            if( ! p_vout->p_sys->b_mouse )
+            {
+                X11TogglePointer( p_vout ); 
+            }
+        }
 #ifdef DEBUG
         /* Other event */
         else
@@ -519,6 +593,17 @@ static int vout_Manage( vout_thread_t *p_vout )
                   p_vout->i_width, p_vout->i_height);
     }
 
+    /* Autohide Cursour */
+    if( mdate() - p_vout->p_sys->i_lastmoved > 2000000 )
+    {
+        /* Hide the mouse automatically */
+        if( p_vout->p_sys->b_mouse )
+        {
+            X11TogglePointer( p_vout ); 
+        }
+    }
+
+    
     return 0;
 }
 
@@ -693,7 +778,8 @@ static int X11CreateWindow( vout_thread_t *p_vout )
 
     XSelectInput( p_vout->p_sys->p_display, p_vout->p_sys->window,
                   StructureNotifyMask | KeyPressMask |
-                  ButtonPressMask | ButtonReleaseMask );
+                  ButtonPressMask | ButtonReleaseMask | 
+                  PointerMotionMask );
 
     if( XDefaultDepth(p_vout->p_sys->p_display, p_vout->p_sys->i_screen) == 8 )
     {
