@@ -4,7 +4,7 @@
  * decoders.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: input.c,v 1.95 2001/04/08 09:04:33 stef Exp $
+ * $Id: input.c,v 1.97 2001/04/12 02:40:09 stef Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -126,12 +126,13 @@ input_thread_t *input_CreateThread ( playlist_item_t *p_item, int *pi_status )
     p_input->stream.i_new_status = p_input->stream.i_new_rate = 0;
     p_input->stream.i_mux_rate = 0;
 
+    /* no stream, no area */
     p_input->stream.i_area_nb = 0;
     p_input->stream.pp_areas = NULL;
+    p_input->stream.p_selected_area = NULL;
     /* By default there is one areas in a stream */
     input_AddArea( p_input );
     p_input->stream.p_selected_area = p_input->stream.pp_areas[0];
-    p_input->stream.p_selected_area->i_seek = NO_SEEK;
 
     /* Initialize stream control properties. */
     p_input->stream.control.i_status = PLAYING_S;
@@ -233,6 +234,7 @@ static void RunThread( input_thread_t *p_input )
 #endif
 
         vlc_mutex_lock( &p_input->stream.stream_lock );
+
         if( p_input->stream.p_selected_area->i_seek != NO_SEEK )
         {
             if( p_input->stream.b_seekable && p_input->pf_seek != NULL )
@@ -254,6 +256,7 @@ static void RunThread( input_thread_t *p_input )
             }
             p_input->stream.p_selected_area->i_seek = NO_SEEK;
         }
+
         vlc_mutex_unlock( &p_input->stream.stream_lock );
 
         i_error = p_input->pf_read( p_input, pp_packets );
