@@ -4,8 +4,10 @@
  * and spawn threads.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
+ * $Id: main.c,v 1.85 2001/04/12 01:52:45 sam Exp $
  *
- * Authors:
+ * Authors: Vincent Seguin <seguin@via.ecp.fr>
+ *          Samuel Hocevar <sam@zoy.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,10 +32,22 @@
 #include <signal.h>                               /* SIGHUP, SIGINT, SIGKILL */
 #include <stdio.h>                                              /* sprintf() */
 
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>                                              /* getopt() */
+#ifdef HAVE_GETOPT_LONG
+#   ifdef HAVE_GETOPT_H
+#       include <getopt.h>                                       /* getopt() */
+#   endif
+#else
+#   include "GNUgetopt/getopt.h"
 #endif
 
+#ifdef SYS_DARWIN1_3
+#   include <mach/mach.h>                               /* Altivec detection */
+#   include <mach/mach_error.h>       /* some day the header files||compiler *
+                                                       will define it for us */
+#   include <mach/bootstrap.h>
+#endif
+
+#include <unistd.h>
 #include <errno.h>                                                 /* ENOMEM */
 #include <stdlib.h>                                  /* getenv(), strtol(),  */
 #include <string.h>                                            /* strerror() */
@@ -50,7 +64,7 @@
 #include "input_ext-intf.h"
 
 #include "intf_msg.h"
-#include "intf_plst.h"
+#include "intf_playlist.h"
 #include "interface.h"
 
 #include "audio_output.h"
@@ -59,9 +73,15 @@
 #include "video_output.h"
 
 #ifdef SYS_BEOS
-#include "beos_specific.h"
+#   include "beos_specific.h"
 #endif
 
+#ifdef SYS_DARWIN1_3
+#   include "darwin_specific.h"
+#endif
+
+#include "netutils.h"                                 /* network_ChannelJoin */
+
 #include "main.h"
 
 /*****************************************************************************
@@ -83,21 +103,19 @@
 #define OPT_FULLSCREEN          165
 #define OPT_OVERLAY             166
 
-#define OPT_VLANS               170
+#define OPT_CHANNELS            170
 #define OPT_SERVER              171
 #define OPT_PORT                172
 #define OPT_BROADCAST           173
 
-#define OPT_AOUT                180
-#define OPT_VOUT                181
-#define OPT_INTF                182
-#define OPT_MOTION              183
-#define OPT_IDCT                184
-#define OPT_YUV                 185
-#define OPT_INPUT               186
+#define OPT_INPUT               180
+#define OPT_MOTION              181
+#define OPT_IDCT                182
+#define OPT_YUV                 183
 
 #define OPT_SYNCHRO             190
 #define OPT_WARNING             191
+#define OPT_VERSION             192
 
 /* Usage fashion */
 #define USAGE                     0
@@ -105,7 +123,6 @@
 #define LONG_HELP                 2
 
 /* Long options */
-#ifdef HAVE_GETOPT_H
 static const struct option longopts[] =
 {
     /*  name,               has_arg,    flag,   val */
@@ -113,21 +130,21 @@ static const struct option longopts[] =
     /* General/common options */
     {   "help",             0,          0,      'h' },
     {   "longhelp",         0,          0,      'H' },
-    {   "version",          0,          0,      'v' },
+    {   "version",          0,          0,      OPT_VERSION },
 
     /* Interface options */
-    {   "intf",             1,          0,      OPT_INTF },
+    {   "intf",             1,          0,      'I' },
     {   "warning",          1,          0,      OPT_WARNING },
 
     /* Audio options */
     {   "noaudio",          0,          0,      OPT_NOAUDIO },
-    {   "aout",             1,          0,      OPT_AOUT },
+    {   "aout",             1,          0,      'A' },
     {   "stereo",           0,          0,      OPT_STEREO },
     {   "mono",             0,          0,      OPT_MONO },
 
     /* Video options */
     {   "novideo",          0,          0,      OPT_NOVIDEO },
-    {   "vout",             1,          0,      OPT_VOUT },
+    {   "vout",             1,          0,      'V' },
     {   "display",          1,          0,      OPT_DISPLAY },
     {   "width",            1,          0,      OPT_WIDTH },
     {   "height",           1,          0,      OPT_HEIGHT },
@@ -140,13 +157,15 @@ static const struct option longopts[] =
     {   "overlay",          0,          0,      OPT_OVERLAY },
 
     /* DVD options */
+    {   "dvdtitle",         1,          0,      't' },
+    {   "dvdchapter",       1,          0,      'T' },
     {   "dvdaudio",         1,          0,      'a' },
     {   "dvdchannel",       1,          0,      'c' },
     {   "dvdsubtitle",      1,          0,      's' },
     
     /* Input options */
     {   "input",            1,          0,      OPT_INPUT },
-    {   "vlans",            0,          0,      OPT_VLANS },
+    {   "channels",         0,          0,      OPT_CHANNELS },
     {   "server",           1,          0,      OPT_SERVER },
     {   "port",             1,          0,      OPT_PORT },
     {   "broadcast",        0,          0,      OPT_BROADCAST },
@@ -157,9 +176,7 @@ static const struct option longopts[] =
 };
 
 /* Short options */
-static const char *psz_shortopts = "hHvga:s:c:";
-#endif
-
+static const char *psz_shortopts = "hHvgt:T:a:s:c:I:A:V:";
 
 /*****************************************************************************
  * Global variable program_data - this is the one and only, see main.h
@@ -169,8 +186,7 @@ main_t *p_main;
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static void SetDefaultConfiguration ( void );
-static int  GetConfiguration        ( int i_argc, char *ppsz_argv[],
+static int  GetConfiguration        ( int *pi_argc, char *ppsz_argv[],
                                       char *ppsz_env[] );
 static int  GetFilenames            ( int i_argc, char *ppsz_argv[] );
 static void Usage                   ( int i_fashion );
@@ -180,6 +196,8 @@ static void InitSignalHandler       ( void );
 static void SimpleSignalHandler     ( int i_signal );
 static void FatalSignalHandler      ( int i_signal );
 
+static int  CPUCapabilities         ( void );
+
 /*****************************************************************************
  * main: parse command line, start interface and spawn threads
  *****************************************************************************
@@ -199,8 +217,8 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     /*
      * System specific initialization code
      */
-#ifdef SYS_BEOS
-    beos_Create();
+#if defined( SYS_BEOS ) || defined( SYS_DARWIN1_3 )
+    system_Create( &i_argc, ppsz_argv, ppsz_env );
 #endif
 
     p_main->i_cpu_capabilities = CPUCapabilities();
@@ -242,26 +260,23 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     /*
      * Read configuration
      */
-    if( GetConfiguration( i_argc, ppsz_argv, ppsz_env ) )  /* parse cmd line */
+    if( GetConfiguration( &i_argc, ppsz_argv, ppsz_env ) )  /* parse cmd line */
     {
         intf_MsgDestroy();
         return( errno );
     }
 
-    p_main->i_warning_level = main_GetIntVariable( INTF_WARNING_VAR,
-                                                   INTF_WARNING_DEFAULT );
-
     /*
      * Initialize playlist and get commandline files
      */
-    p_main->p_playlist = intf_PlstCreate( );
+    p_main->p_playlist = intf_PlaylistCreate( );
     if( !p_main->p_playlist )
     {
         intf_ErrMsg( "playlist error: playlist initialization failed" );
         intf_MsgDestroy();
         return( errno );
     }
-    intf_PlstInit( p_main->p_playlist );
+    intf_PlaylistInit( p_main->p_playlist );
 
     /*
      * Get input filenames given as commandline arguments
@@ -275,7 +290,7 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     if( !p_main->p_bank )
     {
         intf_ErrMsg( "module error: module bank initialization failed" );
-        intf_PlstDestroy( p_main->p_playlist );
+        intf_PlaylistDestroy( p_main->p_playlist );
         intf_MsgDestroy();
         return( errno );
     }
@@ -284,16 +299,13 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     /*
      * Initialize shared resources and libraries
      */
-    /* FIXME: no VLANs */
-#if 0
-    if( p_main->b_vlans && input_VlanCreate() )
+    if( p_main->b_channels && network_ChannelCreate() )
     {
-        /* On error during vlans initialization, switch off vlans */
-        intf_Msg( "Virtual LANs initialization failed : "
-                  "vlans management is deactivated" );
-        p_main->b_vlans = 0;
+        /* On error during Channels initialization, switch off channels */
+        intf_Msg( "Channels initialization failed : "
+                  "Channel management is deactivated" );
+        p_main->b_channels = 0;
     }
-#endif
 
     /*
      * Run interface
@@ -303,7 +315,7 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     {
         intf_ErrMsg( "intf error: interface initialization failed" );
         module_DestroyBank( p_main->p_bank );
-        intf_PlstDestroy( p_main->p_playlist );
+        intf_PlaylistDestroy( p_main->p_playlist );
         intf_MsgDestroy();
         return( errno );
     }
@@ -329,14 +341,14 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     }
 
     /*
-     * Open video device and start aout thread
+     * Open video device and start vout thread
      */
     if( p_main->b_video )
     {
         p_main->p_vout = vout_CreateThread( NULL );
         if( p_main->p_vout == NULL )
         {
-            /* On error during video initialization, switch off audio */
+            /* On error during video initialization, switch off video */
             intf_ErrMsg( "vout error: video initialization failed,"
                          " video is deactivated" );
             p_main->b_video = 0;
@@ -370,15 +382,12 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     }
 
     /*
-     * Free shared resources and libraries
+     * Go back into channel 0 which is the network
      */
-    /* FIXME */
-#if 0
-    if( p_main->b_vlans )
+    if( p_main->b_channels )
     {
-        input_VlanDestroy();
+        network_ChannelJoin( COMMON_CHANNEL );
     }
-#endif
 
     /*
      * Free module bank
@@ -388,13 +397,13 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     /*
      * Free playlist
      */
-    intf_PlstDestroy( p_main->p_playlist );
+    intf_PlaylistDestroy( p_main->p_playlist );
 
-#ifdef SYS_BEOS
     /*
      * System specific cleaning code
      */
-    beos_Destroy();
+#if defined( SYS_BEOS ) || defined( SYS_DARWIN1_3 )
+    system_Destroy();
 #endif
 
     /*
@@ -490,24 +499,6 @@ void main_PutIntVariable( char *psz_name, int i_value )
 
 /* following functions are local */
 
-/*****************************************************************************
- * SetDefaultConfiguration: set default options
- *****************************************************************************
- * This function is called by GetConfiguration before command line is parsed.
- * It sets all the default values required later by the program. At this stage,
- * most structure are not yet allocated, so initialization must be done using
- * environment.
- *****************************************************************************/
-static void SetDefaultConfiguration( void )
-{
-    /*
-     * All features are activated by default except vlans
-     */
-    p_main->b_audio  = 1;
-    p_main->b_video  = 1;
-    p_main->b_vlans  = 0;
-}
-
 /*****************************************************************************
  * GetConfiguration: parse command line
  *****************************************************************************
@@ -517,37 +508,62 @@ static void SetDefaultConfiguration( void )
  * stage, but most structures are not allocated, so only environment should
  * be used.
  *****************************************************************************/
-static int GetConfiguration( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
+static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
 {
-    int c;
-    char * p_pointer;
+    int   i_cmd;
+    char *p_tmp;
 
     /* Set default configuration and copy arguments */
-    p_main->i_argc    = i_argc;
+    p_main->i_argc    = *pi_argc;
     p_main->ppsz_argv = ppsz_argv;
     p_main->ppsz_env  = ppsz_env;
-    SetDefaultConfiguration();
+
+    p_main->b_audio     = 1;
+    p_main->b_video     = 1;
+    p_main->b_channels  = 0;
+
+    p_main->i_warning_level = 4;
 
     /* Get the executable name (similar to the basename command) */
-    p_main->psz_arg0 = p_pointer = ppsz_argv[ 0 ];
-    while( *p_pointer )
+    p_main->psz_arg0 = p_tmp = ppsz_argv[ 0 ];
+    while( *p_tmp )
     {
-        if( *p_pointer == '/' )
+        if( *p_tmp == '/' )
         {
-            p_main->psz_arg0 = ++p_pointer;
+            p_main->psz_arg0 = ++p_tmp;
         }
         else
         {
-            ++p_pointer;
+            ++p_tmp;
         }
     }
 
-    /* Parse command line options */
-#ifdef HAVE_GETOPT_H
-    opterr = 0;
-    while( ( c = getopt_long( i_argc, ppsz_argv, psz_shortopts, longopts, 0 ) ) != EOF )
+#ifdef SYS_DARWIN1_3
+    /* When vlc.app is run by double clicking in Mac OS X, the 2nd arg
+     * is the PSN - process serial number (a unique PID-ish thingie)
+     * still ok for real Darwin & when run from command line */
+    if ( strncmp( ppsz_argv[ 1 ] , "-psn" , 4) == 0 )
+                                  /* for example -psn_0_9306113 */
     {
-        switch( c )
+        /* GDMF!... I can't do this or else the MacOSX window server will
+         * not pick up the PSN and not register the app and we crash...
+         * hence the following kludge otherwise we'll get confused w/ argv[1]
+         * being an input file name */
+#if 0
+        ppsz_argv[ 1 ] = NULL;
+#endif
+        *pi_argc = *pi_argc - 1;
+        pi_argc--;
+        return( 0 );
+    }
+#endif
+
+    /* Parse command line options */
+    opterr = 0;
+    while( ( i_cmd = getopt_long( *pi_argc, ppsz_argv,
+                                   psz_shortopts, longopts, 0 ) ) != EOF )
+    {
+        switch( i_cmd )
         {
         /* General/common options */
         case 'h':                                              /* -h, --help */
@@ -558,24 +574,28 @@ static int GetConfiguration( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
             Usage( LONG_HELP );
             return( -1 );
             break;
-        case 'v':                                           /* -v, --version */
+        case OPT_VERSION:                                       /* --version */
             Version();
             return( -1 );
             break;
+        case 'v':                                           /* -v, --verbose */
+            p_main->i_warning_level--;
+            break;
 
         /* Interface warning messages level */
-        case OPT_INTF:                                             /* --intf */
+        case 'I':                                              /* -I, --intf */
             main_PutPszVariable( INTF_METHOD_VAR, optarg );
             break;
         case OPT_WARNING:                                       /* --warning */
-            main_PutIntVariable( INTF_WARNING_VAR, atoi(optarg) );
+            intf_ErrMsg( "intf error: `--warning' is deprecated, use `-v'" );
+            p_main->i_warning_level = atoi(optarg);
             break;
 
         /* Audio options */
         case OPT_NOAUDIO:                                       /* --noaudio */
             p_main->b_audio = 0;
             break;
-        case OPT_AOUT:                                             /* --aout */
+        case 'A':                                              /* -A, --aout */
             main_PutPszVariable( AOUT_METHOD_VAR, optarg );
             break;
         case OPT_STEREO:                                         /* --stereo */
@@ -589,7 +609,7 @@ static int GetConfiguration( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_NOVIDEO:                                       /* --novideo */
             p_main->b_video = 0;
             break;
-        case OPT_VOUT:                                             /* --vout */
+        case 'V':                                              /* -V, --vout */
             main_PutPszVariable( VOUT_METHOD_VAR, optarg );
             break;
         case OPT_DISPLAY:                                       /* --display */
@@ -624,6 +644,12 @@ static int GetConfiguration( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
             break;
 
         /* DVD options */
+        case 't':
+            main_PutIntVariable( INPUT_TITLE_VAR, atoi(optarg) );
+            break;
+        case 'T':
+            main_PutIntVariable( INPUT_CHAPTER_VAR, atoi(optarg) );
+            break;
         case 'a':
             if ( ! strcmp(optarg, "ac3") )
                 main_PutIntVariable( INPUT_AUDIO_VAR, REQUESTED_AC3 );
@@ -645,8 +671,8 @@ static int GetConfiguration( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_INPUT:                                           /* --input */
             main_PutPszVariable( INPUT_METHOD_VAR, optarg );
             break;
-        case OPT_VLANS:                                           /* --vlans */
-            p_main->b_vlans = 1;
+        case OPT_CHANNELS:                                     /* --channels */
+            p_main->b_channels = 1;
             break;
         case OPT_SERVER:                                         /* --server */
             main_PutPszVariable( INPUT_SERVER_VAR, optarg );
@@ -666,13 +692,19 @@ static int GetConfiguration( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
         /* Internal error: unknown option */
         case '?':
         default:
-            intf_ErrMsg( "intf error: unknown option `%s'", ppsz_argv[optind - 1] );
+            intf_ErrMsg( "intf error: unknown option `%s'",
+                         ppsz_argv[optind - 1] );
             Usage( USAGE );
             return( EINVAL );
             break;
         }
     }
-#endif
+
+    if( p_main->i_warning_level < 0 )
+    {
+        p_main->i_warning_level = 0;
+    }
+
     return( 0 );
 }
 
@@ -688,7 +720,8 @@ static int GetFilenames( int i_argc, char *ppsz_argv[] )
     /* We assume that the remaining parameters are filenames */
     for( i_opt = optind; i_opt < i_argc; i_opt++ )
     {
-        intf_PlstAdd( p_main->p_playlist, PLAYLIST_END, ppsz_argv[ i_opt ] );
+        intf_PlaylistAdd( p_main->p_playlist, PLAYLIST_END,
+                          ppsz_argv[ i_opt ] );
     }
 
     return( 0 );
@@ -714,15 +747,15 @@ static void Usage( int i_fashion )
 
     /* Options */
     intf_MsgImm( "\nOptions:"
-          "\n      --intf <module>            \tinterface method"
-          "\n      --warning <level>          \tdisplay warning messages"
+          "\n  -I, --intf <module>            \tinterface method"
+          "\n  -v, --verbose                  \tverbose mode (cumulative)"
           "\n"
           "\n      --noaudio                  \tdisable audio"
-          "\n      --aout <module>            \taudio output method"
+          "\n  -A, --aout <module>            \taudio output method"
           "\n      --stereo, --mono           \tstereo/mono audio"
           "\n"
           "\n      --novideo                  \tdisable video"
-          "\n      --vout <module>            \tvideo output method"
+          "\n  -V, --vout <module>            \tvideo output method"
           "\n      --display <display>        \tdisplay string"
           "\n      --width <w>, --height <h>  \tdisplay dimensions"
           "\n  -g, --grayscale                \tgrayscale output"
@@ -734,29 +767,30 @@ static void Usage( int i_fashion )
           "\n      --yuv <module>             \tYUV method"
           "\n      --synchro <type>           \tforce synchro algorithm"
           "\n"
+          "\n  -t, --dvdtitle <num>           \tchoose DVD title"
+          "\n  -T, --dvdchapter <num>         \tchoose DVD chapter"
           "\n  -a, --dvdaudio <type>          \tchoose DVD audio type"
           "\n  -c, --dvdchannel <channel>     \tchoose DVD audio channel"
           "\n  -s, --dvdsubtitle <channel>    \tchoose DVD subtitle channel"
           "\n"
           "\n      --input                    \tinput method"
-          "\n      --vlans                    \tenable vlans"
+          "\n      --channels                    \tenable channels"
           "\n      --server <host>            \tvideo server address"
           "\n      --port <port>              \tvideo server port"
           "\n      --broadcast                \tlisten to a broadcast"
           "\n"
           "\n  -h, --help                     \tprint help and exit"
           "\n  -H, --longhelp                 \tprint long help and exit"
-          "\n  -v, --version                  \toutput version information and exit" );
+          "\n      --version                  \toutput version information and exit" );
 
     if( i_fashion == SHORT_HELP )
         return;
 
     /* Interface parameters */
-    intf_MsgImm( "\nInterface parameters:\n"
+    intf_MsgImm( "\nInterface parameters:"
         "\n  " INTF_METHOD_VAR "=<method name>          \tinterface method"
-        "\n  " INTF_INIT_SCRIPT_VAR "=<filename>             \tinitialization script"
-        "\n  " INTF_CHANNELS_VAR "=<filename>            \tchannels list"
-        "\n  " INTF_WARNING_VAR "=<level>                \twarning level" );
+        "\n  " INTF_INIT_SCRIPT_VAR "=<filename>               \tinitialization script"
+        "\n  " INTF_CHANNELS_VAR "=<filename>            \tchannels list" );
 
     /* Audio parameters */
     intf_MsgImm( "\nAudio parameters:"
@@ -783,18 +817,20 @@ static void Usage( int i_fashion )
     /* DVD parameters */
     intf_MsgImm( "\nDVD parameters:"
         "\n  " INPUT_DVD_DEVICE_VAR "=<device>           \tDVD device"
-        "\n  " INPUT_AUDIO_VAR "={ac3|lpcm|mpeg|off}     \taudio type"
-        "\n  " INPUT_CHANNEL_VAR "=[0-15]                \taudio channel"
-        "\n  " INPUT_SUBTITLE_VAR "=[0-31]               \tsubtitle channel" );
+        "\n  " INPUT_TITLE_VAR "=<title>             \ttitle number"
+        "\n  " INPUT_CHAPTER_VAR "=<chapter>         \tchapter number"
+        "\n  " INPUT_AUDIO_VAR "={ac3|lpcm|mpeg|off} \taudio type"
+        "\n  " INPUT_CHANNEL_VAR "=[0-15]            \taudio channel"
+        "\n  " INPUT_SUBTITLE_VAR "=[0-31]           \tsubtitle channel" );
 
     /* Input parameters */
-    intf_MsgImm( "\nInput parameters:\n"
+    intf_MsgImm( "\nInput parameters:"
         "\n  " INPUT_SERVER_VAR "=<hostname>          \tvideo server"
         "\n  " INPUT_PORT_VAR "=<port>            \tvideo server port"
         "\n  " INPUT_IFACE_VAR "=<interface>          \tnetwork interface"
         "\n  " INPUT_BROADCAST_VAR "={1|0}            \tbroadcast mode"
-        "\n  " INPUT_VLAN_SERVER_VAR "=<hostname>     \tvlan server"
-        "\n  " INPUT_VLAN_PORT_VAR "=<port>           \tvlan server port" );
+        "\n  " INPUT_CHANNEL_SERVER_VAR "=<hostname>     \tchannel server"
+        "\n  " INPUT_CHANNEL_PORT_VAR "=<port>         \tchannel server port" );
 
 }
 
@@ -867,3 +903,138 @@ static void FatalSignalHandler( int i_signal )
     p_main->p_intf->b_die = 1;
 }
 
+/*****************************************************************************
+ * CPUCapabilities: list the processors MMX support and other capabilities
+ *****************************************************************************
+ * This function is called to list extensions the CPU may have.
+ *****************************************************************************/
+static int CPUCapabilities( void )
+{
+    int i_capabilities = CPU_CAPABILITY_NONE;
+
+#if defined( SYS_BEOS )
+    i_capabilities |= CPU_CAPABILITY_486
+                      | CPU_CAPABILITY_586
+                      | CPU_CAPABILITY_MMX;
+
+#elif defined( SYS_DARWIN1_3 )
+
+    struct host_basic_info hi;
+    kern_return_t          ret;
+    host_name_port_t       host;
+
+    int i_size;
+    char *psz_name, *psz_subname;
+
+    /* Should 'never' fail? */
+    host = mach_host_self();
+
+    i_size = sizeof( hi ) / sizeof( int );
+    ret = host_info( host, HOST_BASIC_INFO, ( host_info_t )&hi, &i_size );
+
+    if( ret != KERN_SUCCESS )
+    {
+        intf_ErrMsg( "error: couldn't get CPU information" );
+        return( i_capabilities );
+    }
+
+    slot_name( hi.cpu_type, hi.cpu_subtype, &psz_name, &psz_subname );
+    /* FIXME: need better way to detect newer proccessors.
+     * could do strncmp(a,b,5), but that's real ugly */
+    if( strcmp(psz_name, "ppc7400") || strcmp(psz_name, "ppc7450") )
+    {
+        i_capabilities |= CPU_CAPABILITY_ALTIVEC;
+    }
+
+#elif defined( __i386__ )
+    unsigned int  i_eax, i_ebx, i_ecx, i_edx;
+    boolean_t     b_amd;
+
+#   define cpuid( a )              \
+    asm volatile ( "cpuid"         \
+                 : "=a" ( i_eax ), \
+                   "=b" ( i_ebx ), \
+                   "=c" ( i_ecx ), \
+                   "=d" ( i_edx )  \
+                 : "a"  ( a )      \
+                 : "cc" );         \
+
+    /* test for a 486 CPU */
+    asm volatile ( "pushfl\n\t"
+                   "popl %%eax\n\t"
+                   "movl %%eax, %%ebx\n\t"
+                   "xorl $0x200000, %%eax\n\t"
+                   "pushl %%eax\n\t"
+                   "popfl\n\t"
+                   "pushfl\n\t"
+                   "popl %%eax"
+                 : "=a" ( i_eax ),
+                   "=b" ( i_ebx )
+                 :
+                 : "cc" );
+
+    if( i_eax == i_ebx )
+    {
+        return( i_capabilities );
+    }
+
+    i_capabilities |= CPU_CAPABILITY_486;
+
+    /* the CPU supports the CPUID instruction - get its level */
+    cpuid( 0x00000000 );
+
+    if( !i_eax )
+    {
+        return( i_capabilities );
+    }
+
+    /* FIXME: this isn't correct, since some 486s have cpuid */
+    i_capabilities |= CPU_CAPABILITY_586;
+
+    /* borrowed from mpeg2dec */
+    b_amd = ( i_ebx == 0x68747541 ) && ( i_ecx == 0x444d4163 )
+                    && ( i_edx == 0x69746e65 );
+
+    /* test for the MMX flag */
+    cpuid( 0x00000001 );
+
+    if( ! (i_edx & 0x00800000) )
+    {
+        return( i_capabilities );
+    }
+
+    i_capabilities |= CPU_CAPABILITY_MMX;
+
+    if( i_edx & 0x02000000 )
+    {
+        i_capabilities |= CPU_CAPABILITY_MMXEXT;
+    }
+    
+    /* test for additional capabilities */
+    cpuid( 0x80000000 );
+
+    if( i_eax < 0x80000001 )
+    {
+        return( i_capabilities );
+    }
+
+    /* list these additional capabilities */
+    cpuid( 0x80000001 );
+
+    if( i_edx & 0x80000000 )
+    {
+        i_capabilities |= CPU_CAPABILITY_3DNOW;
+    }
+
+    if( b_amd && ( i_edx & 0x00400000 ) )
+    {
+        i_capabilities |= CPU_CAPABILITY_MMXEXT;
+    }
+
+#else
+    /* default behaviour */
+
+#endif
+    return( i_capabilities );
+}
+
